import { FlatGitignoreOptions } from 'eslint-config-flat-gitignore';
import { ParserOptions } from '@typescript-eslint/parser';
import { FlatESLintConfigItem, MergeIntersection, RenamePrefix, TypeScriptRules, VitestRules, NRules, Prefix, ReactHooksRules, ReactRules, JSDocRules, ImportRules, EslintRules, JsoncRules, VueRules, UnicornRules, EslintCommentsRules, RuleConfig } from '@antfu/eslint-define-config';
import { Linter } from 'eslint';
import { Rules } from 'eslint-plugin-antfu';
import { RequiredOptions, LiteralUnion, BuiltInParserName } from 'prettier';

type PrettierCustomParser = 'astro' | 'svelte' | 'toml';
type PrettierParser = BuiltInParserName | PrettierCustomParser;
interface PrettierOptions extends RequiredOptions {
    parser: LiteralUnion<PrettierParser>;
}
type PartialPrettierExtendedOptions = Partial<PrettierOptions>;
type WrapRuleConfig<T extends {
    [key: string]: any;
}> = {
    [K in keyof T]: T[K] extends RuleConfig ? T[K] : RuleConfig<T[K]>;
};
type EslintFlatRules = WrapRuleConfig<MergeIntersection<RenamePrefix<TypeScriptRules, '@typescript-eslint/', 'ts/'> & RenamePrefix<VitestRules, 'vitest/', 'test/'> & RenamePrefix<NRules, 'n/', 'node/'> & Prefix<Rules, 'antfu/'> & ReactHooksRules & ReactRules & JSDocRules & ImportRules & EslintRules & JsoncRules & VueRules & UnicornRules & EslintCommentsRules & {
    'test/no-only-tests': RuleConfig<[]>;
}>>;
type FlatConfigItem = Omit<FlatESLintConfigItem<EslintFlatRules, false>, 'plugins'> & {
    /**
     * Custom name of each config item
     */
    name?: string;
    /**
     * An object containing a name-value mapping of plugin names to plugin objects. When `files` is specified, these plugins are only available to the matching files.
     *
     * @see [Using plugins in your configuration](https://eslint.org/docs/latest/user-guide/configuring/configuration-files-new#using-plugins-in-your-configuration)
     */
    plugins?: Record<string, any>;
};
type UserConfigItem = FlatConfigItem | Linter.FlatConfig;

type Awaitable<T> = T | Promise<T>;
type OptionsTypescript = (OptionsTypeScriptWithTypes & OptionsOverrides) | (OptionsTypeScriptParserOptions & OptionsOverrides);
interface OptionsFormatters {
    /**
     * Enable formatting support for HTML.
     */
    html?: boolean;
    /**
     * Enable formatting support for CSS, Less, Sass, and SCSS.
     */
    css?: boolean;
    /**
     * Enable formatting support for Markdown.
     */
    markdown?: boolean;
    /**
     * Enable formatting support for GraphQL.
     */
    graphql?: boolean;
    /**
     * Enable formatting support for Yaml.
     */
    yaml?: boolean;
    /**
     * Enable formatting support for Toml.
     */
    toml?: boolean;
}
interface OptionsComponentExts {
    /**
     * Additional extensions for components.
     *
     * @example ['vue']
     * @default []
     */
    componentExts?: string[];
}
interface OptionsTypeScriptParserOptions {
    /**
     * Additional parser options for TypeScript.
     */
    parserOptions?: Partial<ParserOptions>;
    /**
     * Glob patterns for files that should be type aware.
     * @default ['**\/*.{ts,tsx}']
     */
    filesTypeAware?: string[];
}
interface OptionsTypeScriptWithTypes {
    /**
     * When this options is provided, type aware rules will be enabled.
     * @see https://typescript-eslint.io/linting/typed-linting/
     */
    tsconfigPath?: string | string[];
}
interface OptionsOverrides {
    overrides?: FlatConfigItem['rules'];
}
interface OptionsUnoCSS extends OptionsOverrides {
    /**
     * Enable attributify support.
     * @default true
     */
    attributify?: boolean;
    /**
     * Enable strict mode by throwing errors about blocklisted classes.
     * @default false
     */
    strict?: boolean;
}
interface OptionsVue extends OptionsOverrides {
    /**
     * The vue version
     *
     * @default 3
     */
    version?: 2 | 3;
}
interface OptionsConfig extends OptionsComponentExts {
    /**
     * The current working directory
     *
     * @default process.cwd()
     */
    cwd?: string;
    /**
     * Enable gitignore support.
     *
     * Passing an object to configure the options.
     *
     * @see https://github.com/antfu/eslint-config-flat-gitignore
     * @default true
     */
    gitignore?: boolean | FlatGitignoreOptions;
    /**
     * Core rules. Can't be disabled.
     */
    javascript?: OptionsOverrides;
    /**
     * Enable TypeScript support.
     *
     * Passing an object to enable TypeScript Language Server support.
     *
     * @default auto-detect based on the dependencies
     */
    typescript?: boolean | OptionsTypescript;
    /**
     * Enable test support.
     *
     * @default true
     */
    test?: boolean | OptionsOverrides;
    /**
     * Enable Vue support.
     *
     * @default auto-detect based on the dependencies
     */
    vue?: boolean | OptionsVue;
    /**
     * Enable JSONC support.
     *
     * @default true
     */
    jsonc?: boolean | OptionsOverrides;
    /**
     * Enable react rules.
     *
     * Requires installing:
     * - `eslint-plugin-react`
     * - `eslint-plugin-react-hooks`
     * - `eslint-plugin-react-refresh`
     *
     * @default false
     */
    react?: boolean | OptionsOverrides;
    /**
     * Enable svelte rules.
     *
     * Requires installing:
     * - `eslint-plugin-svelte`
     *
     * @default false
     */
    svelte?: boolean | OptionsOverrides;
    /**
     * Enable unocss rules.
     *
     * Requires installing:
     * - `@unocss/eslint-plugin`
     *
     * @default false
     */
    unocss?: boolean | OptionsUnoCSS;
    /**
     * Whether to use prettierrc
     *
     * If true, the rules in prettierrc will override the default rules
     *
     * @default true
     */
    usePrettierrc?: boolean;
    /**
     * Use external formatters to format files.
     *
     * Requires installing:
     * - `eslint-plugin-prettier`
     *
     * @default
     * {
     *  "html": true,
     *  "css": true,
     *  "graphql": false,
     *  "markdown": false
     *  "yaml": false
     *  "toml": false
     * }
     */
    formatter?: OptionsFormatters;
    /**
     * Default prettier rules
     *
     * @default
     * ```json
     * {
     *   "arrowParens": "avoid",
     *   "htmlWhitespaceSensitivity": "ignore"
     *   "printWidth": 80,
     *   "semi": false,
     *   "singleQuote": true,
     * }
     * ```
     */
    prettierRules?: PartialPrettierExtendedOptions;
    /**
     * Control to disable some rules in editors.
     * @default auto-detect based on the process.env
     */
    isInEditor?: boolean;
    /**
     * Provide overrides for rules for each integration.
     *
     * @deprecated use `overrides` option in each integration key instead
     */
    overrides?: {
        javascript?: FlatConfigItem['rules'];
        typescript?: FlatConfigItem['rules'];
        test?: FlatConfigItem['rules'];
        vue?: FlatConfigItem['rules'];
        jsonc?: FlatConfigItem['rules'];
        react?: FlatConfigItem['rules'];
        svelte?: FlatConfigItem['rules'];
    };
}

/**
 * Construct an array of ESLint flat config items.
 */
declare function defineConfig(options?: OptionsConfig & FlatConfigItem, ...userConfigs: Awaitable<UserConfigItem | UserConfigItem[]>[]): Promise<UserConfigItem[]>;

export { defineConfig };
